/*		
	flip~						
*/

// ---------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------

#include "ext.h"							
#include "ext_obex.h"
#include "z_dsp.h"
#include "buffer.h"
#include "ext_globalsymbol.h"
#include "ext_atomic.h"

// ---------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------
#pragma mark class

typedef struct _flip {
	t_pxobject	ob;	
	t_buffer	*buf;
	t_symbol	*sym;
	} t_flip;

// ---------------------------------------------------------------------------------------------------------------

void		*flip_new		(t_symbol *s, long argc, t_atom *argv);
void		flip_free		(t_flip *x);
void		flip_assist		(t_flip *x, void *b, long m, long a, char *s);

void		flip_dsp		(t_flip *x, t_signal **sp, short *count);
t_int		*dsp_perform	(t_int *w);

void		flip_set		(t_flip *x, t_symbol *s);
t_max_err	flip_notify		(t_flip *x, t_symbol *s, t_symbol *msg, void *sender, void *data);
void		flip_dblclick	(t_flip *x);

void		flip_bang		(t_flip *x); 

// ---------------------------------------------------------------------------------------------------------------

static t_class *flip_class;

int main (void)
{	
	t_class *c = NULL;
	
	c = class_new ("flip~", (method)flip_new, (method)flip_free, (long)sizeof(t_flip), 0L, A_GIMME, 0);
	
	class_dspinit (c);
	
	class_addmethod (c, (method)flip_dsp,		"dsp", A_CANT, 0); 
	class_addmethod	(c, (method)flip_assist,	"assist", A_CANT, 0);
	class_addmethod (c, (method)flip_set,		"set", A_SYM, 0);
	class_addmethod (c, (method)flip_notify,	"notify", A_CANT, 0);
	class_addmethod (c, (method)flip_dblclick,	"dblclick", A_CANT, 0);
	class_addmethod (c, (method)flip_bang,		"bang", 0);

	class_register (CLASS_BOX, c); 
	flip_class = c;
	
	return 0;
}

// ---------------------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------------------
#pragma mark instance

void *flip_new (t_symbol *s, long argc, t_atom *argv)
{
	t_flip *x = NULL;

	if (x = (t_flip *)object_alloc(flip_class)) 
		{
			x->buf		= NULL;
			x->sym		= NULL;
			
			if (argc && atom_gettype (argv) == A_SYM)
				{
					flip_set (x, atom_getsym (argv));	
				}
			
			outlet_new ((t_object *)x, "signal");
			dsp_setup ((t_pxobject *)x, 1);
		}
			
	return x;
}

void flip_free (t_flip *x)
{ 
	if (x->sym)
		{
			globalsymbol_dereference ((t_object *)x, x->sym->s_name, "buffer~");
		}
		
	dsp_free ((t_pxobject *)x);
}

void flip_assist (t_flip *x, void *b, long m, long a, char *s)
{
	if (m == ASSIST_INLET) 
		{ 
			sprintf (s, "(signal)");
		} 
	else 
		{	
			sprintf (s, "(signal)");
		}
}

// ---------------------------------------------------------------------------------------------------------------	
// ---------------------------------------------------------------------------------------------------------------	
#pragma mark signalProcess

void flip_dsp (t_flip *x, t_signal **sp, short *count)
{
	dsp_add (dsp_perform, 4, x, sp[0]->s_vec, sp[0]->s_n, sp[1]->s_vec);
}

t_int *dsp_perform (t_int *w)
{
	t_flip *x = (t_flip *)w[1];
	t_float *in = (t_float *)w[2];
	long n = (long)w[3];
	t_float *out = (t_float *)w[4];
	
	long i;
	
	if (x->ob.z_disabled) 
		{
			return w + 5;
		}
	
	for (i = 0; i < n; i ++)
		{								 	
			out[i] = in[i];							
		}	
	
	return w + 5;
}

// ---------------------------------------------------------------------------------------------------------------	
// ---------------------------------------------------------------------------------------------------------------	
#pragma mark bindToBuffer

void flip_set (t_flip *x, t_symbol *s)
{
	if (s != x->sym)
		{
			x->buf = (t_buffer *)globalsymbol_reference ((t_object *)x, s->s_name, "buffer~");
			
			if (x->sym)
				{
					globalsymbol_dereference ((t_object *)x, x->sym->s_name, "buffer~");
				}
				
			x->sym = s;
		}
}

t_max_err flip_notify (t_flip *x, t_symbol *s, t_symbol *msg, void *sender, void *data)
{
	if (msg == gensym ("globalsymbol_binding"))
		{
			x->buf = (t_buffer *)x->sym->s_thing;
		}
	else if (msg == gensym ("globalsymbol_unbinding"))
		{
			x->buf = NULL;
		}
	
	return MAX_ERR_NONE;
}

void flip_dblclick (t_flip *x)
{
	if (x->buf)
		{
			mess0 ((t_object *)x->buf, gensym ("dblclick"));
		}
}

// ---------------------------------------------------------------------------------------------------------------	
// ---------------------------------------------------------------------------------------------------------------	
#pragma mark copySignal

void flip_bang (t_flip *x) 
{
	if (x->buf)
		{	
			t_buffer	*b = x->buf;
			float		*copy = NULL;
			
			if (copy = (float *)sysmem_newptrclear (sizeof(float) * b->b_frames))
				{
					
					ATOMIC_INCREMENT((int32_t*)&b->b_inuse);
				
					if (b->b_valid)
						{
							long i;
							long k = 0; // channel you want
							
							for (i = 0; i < b->b_frames; i++)
								{
									copy[i] = *(b->b_samples + k);
									post ("%ld : %f", i, copy[i]);
									k += b->b_nchans;
								}
						}	
					
					ATOMIC_DECREMENT((int32_t*)&b->b_inuse);
					
					sysmem_freeptr (copy);
				}
		}
}

// ---------------------------------------------------------------------------------------------------------------	
// -------------------------------------------------------------------------------------------------------------:x